<!DOCTYPE html>
<html lang="{{ config['DEFAULT_LANGUAGE'] }}">
<head>
    <meta charset="UTF-8">
    <title>{{ page.title }}</title>

    <meta name="description" content="{{ page.description }}">

    <meta name="viewport" content="width=device-width,initial-scale=1">
    <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
    <meta name="HandheldFriendly" content="true">

    {% block meta %}{% endblock %}

    <link rel="canonical" href="{{ config['CANONICAL_URL'] }}">

    <link rel="shortcut icon" type="image/png" href="{{ elemental_url_for_static('app/images/favicon.ico') }}"/>

    {% block styles %}{% endblock %}

</head>
<body>

    {% block content %}{% endblock %}
    {% block scripts %}{% endblock %}

</body>
</html>